/*
 * Calendar.java
 * 
 * Created on 02. Dezember 2006, 20:41
 *
 */

package net.lin2win.calendar;

import com.sun.xml.registry.common.tools.bindings_v3.Result;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedList;
import java.util.UUID;
import javax.annotation.Resource;
import javax.ejb.Stateless;
import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebService;
import javax.sql.DataSource;

/**
 * Web Service Class for the Calendar Project.
 * Each call is stateless so you have to provide username and password.
 * For security issues the return value of some method is false or empty.
 * That means the operantion was successful or was not.
 * The attacker shall not see a stack trace. <br>
 * <br>
 * <b>Hint for Admins:</b> <br>
 * If you use this service class ofer a public network ensure that you use encryption ! 
 * Lookup the Application Server Manual for proper configuration.
 * @author Markus Nbauer
 */

@Stateless()
@WebService()
public class Calendar {

    @Resource(name = "Calendar")
    private DataSource calendar;
    private String dbUser = "app";
    private String dbPass = "app123";
    private String dbDriver = "sun.jdbc.odbc.JdbcOdbcDriver";
    private String dbUrl = "jdbc:odbc:Calendar";
    
    /**
     * Create a new User Account for the Calendar Service.
     * Username must be unique, password must be at least 3 Characters.
     * A unique user ID is generated automatically.
     * @return True if Creation was successful, FALSE if username already exists or operation failed.
     */
    @WebMethod
    public boolean createUser(@WebParam(name = "username") final String username, @WebParam(name = "password") final String password) {
        
            // Username is mandatory, Password must be longer than 3 chars
            if(!username.equals("") && password.length()>=3) {
                try {
                    Class.forName(dbDriver);   
                    Connection con = DriverManager.getConnection(dbUrl,dbUser,dbPass);
                    
                    // Lookup if username is in use
                    Statement stat = con.createStatement();
                    ResultSet result = stat.executeQuery("SELECT COUNT(username) AS ct FROM users WHERE username='"+username+"';");
                    result.next();
                    if(result.getInt("ct")==0) {
                    
                        // generate unique universally Identifier for user ID
                        UUID guid = UUID.randomUUID();
                        String userID = guid.toString().replaceAll("-","");
                        stat = con.createStatement();
                        stat.execute("insert into users (USERID,USERNAME,PASSWORD) values ('"+userID+"','"+username+"','"+password+"');");
                        con.commit();
                        con.close();
                        
                        // INSERT was successful, return TRUE
                        return true;
                        
                     } else {
                        return false;
                     }
                // Operation failed, return FALSE    
                } catch (SQLException ex) {
                    ex.printStackTrace();
                    return false;
                } catch(ClassNotFoundException ex) {
                    ex.printStackTrace();
                    return false;
                }
            } else {
                // Invalid Username or Password, return FALSE
                return false;
            }
    }

    
    /**
     * Delete a User Account from the Calendar Service.
     * All Appointments for this user will be deleted too.
     * Authentication with Username and Password is required.
     * @return True if Delete Operation was Successful.
     */
    @WebMethod
    public boolean deleteUser(@WebParam(name = "username") final String username, @WebParam(name = "password") final String password) {
        
        Connection con;
        Statement stat;
        ResultSet   result;
        String  userId;
        int numRows;
        
        try {
           
            Class.forName(dbDriver);
            con = DriverManager.getConnection(dbUrl,dbUser,dbPass);
            
            // Find User
            stat = con.createStatement();
            result = stat.executeQuery("SELECT userid FROM users WHERE username='"+username+"' AND password='"+password+"';");
            if(result.next()) {
                userId = result.getString("USERID");
            } else {
                return false;
            }
            
            // Delete all Appointments for this user
            stat = con.createStatement();
            numRows = stat.executeUpdate("DELETE FROM appointment WHERE userid='"+userId+"';");
            
            // Delete the User
            stat = con.createStatement();
            int rows = stat.executeUpdate("DELETE FROM users WHERE username='"+username+"' AND password='"+password+"';");
            con.commit();
            con.close();
            
            // TRUE if a row was affected, FALSE if no row was deleted cause username or password were not correct
            return rows>0;
        } catch(SQLException ex) {
            ex.printStackTrace();
            return false;
        } catch(ClassNotFoundException ex) {
            ex.printStackTrace();
            return false;
        }
    }
    
    /**
     * Create a new Appointment.
     * Authentication with username and password is required.
     * You must also provide a valid title, time and Appointment Type ID.
     * Use the Web Method getTypes to get Appointment Type IDs and Name
     *
     * @param username A valid username like "Hans".
     * @param password The user password like "123456".
     * @param title A short heading for your appointment like "Meeting with Mr.Smith".
     * @param description A longtext description for your apppointment. An empty String is allowed here "".
     * @param time Time in Miliseconds since 1.1.1970 UTC when you appointment takes place. 
     * @param type A type ID like "1a5a46f04704433093d09431c51893ad" . User getTypes to get Type IDs and Name.
     * @param priority Integer Value.
     * @return Appointment ID if creation was successful otherwise a Error String starting with "ERROR:"
     */
    @WebMethod
    public String createAppointment(@WebParam(name = "username") final String username, @WebParam(name = "password") final String password, @WebParam(name = "title") final String title, @WebParam(name = "description") final String description, @WebParam(name = "time") final String time, @WebParam(name = "type") String type, @WebParam(name = "priority") final int priority) {
        try {
            if(username!=null && password!=null && title!=null && time!=null && type!=null &&
               !username.equals("") && !password.equals("") && !title.equals("") && !time.equals("") && !type.equals("")) {
                
                String appId;               // unique appointment ID
                String userId;              // user ID
                String appTitle;            // appointment title like "meeting with Mr.Smith"
                String appDescription = ""; // long text description
                String appDate;             // appointment date
                String appType;             // type from TYPE table
                boolean accepted = true;    // if appointment was accepted by user, by default true for creating user
                Statement   stat;           // SQL Statements
                ResultSet   result;         // Lookup Result
                
                // Load Driver and create Connection
                Class.forName(dbDriver);
                Connection con = DriverManager.getConnection(dbUrl,dbUser,dbPass);
                
                // Create new Appointment ID
                UUID uuid = UUID.randomUUID();
                appId = uuid.toString().replaceAll("-","");
                
                // Lookup if User exists and get User ID
                // Username and Password must be valid
                stat = con.createStatement();
                result = stat.executeQuery("SELECT userid FROM users WHERE username='"+username+"' AND password='"+password+"';");
                if(result.next()) {
                    userId = result.getString("USERID");
                } else {
                    return "ERROR:Username or Password not correct";
                }
                
                // Title comes from params
                appTitle = title;
                
                // Description comes from params or is empty
                if(description==null || description.equals(""))
                    appDescription = "No Description";
                else
                    appDescription = description;
                
                // Date comes from params
                appDate = time;
                
                java.sql.Date sqlDate = new java.sql.Date(Long.valueOf(time));
                // INSERT                
//                System.out.println("INSERT INSERT INSERT");
//                System.out.println(appId+","+userId+","+appTitle+","+appDescription+","+sqlDate.toString()+","+type+","+1+","+priority);
                
                stat.execute("INSERT INTO appointment(APPID,USERID,TITLE,DESCRIPTION,APPDATE,TYPE,ACCEPTED,PRIORITY) VALUES('"+appId+"','"+userId+"','"+appTitle+"','"+appDescription+"',"+sqlDate.toString()+",'"+type+"',1,"+priority+");");
                con.commit();
                con.close();
                
                // INSERT was successful, return Appointment ID
                return appId;
            } else {
                // Missing Params, return empty String
                return "ERROR:Missing Parameters USERNAME,PASSWORD,TITLE,TIME,TYPE are mandatory";
            }
        } catch(Exception ex) {
            return "ERROR:"+ex.toString();
        }
       
    }


    /**
     * Web service operation
     */
    @WebMethod
    public String[][] getAppointments(@WebParam(name = "username") final String username, @WebParam(name = "password") final String password) {
        
        Connection      con;
        Statement       stat;
        ResultSet       result;
        String          userId;
        ArrayList       list = new ArrayList();
        String[]          appointment;
        
        try {
            Class.forName(dbDriver);
            con = DriverManager.getConnection(dbUrl,dbUser,dbPass);
            
            stat = con.createStatement();
            result = stat.executeQuery("SELECT userid FROM users WHERE username='"+username+"' AND password='"+password+"';");
            if(result.next()) {
                userId = result.getString("USERID");
                
                stat = con.createStatement();
                result = stat.executeQuery("SELECT * FROM appointment WHERE userid='"+userId+"';");
                while(result.next()) {
                    appointment = new String[5];
                    appointment[0] = result.getString("APPID");
                    appointment[1] = result.getString("TITLE");
                    appointment[2] = result.getString("DESCRIPTION");
                    appointment[3] = result.getDate("APPDATE").toString();
                    appointment[4] = result.getString("TYPE");

                    list.add(appointment);
                }
            }
            
            int num = list.size();
            String[][] apps = new String[num][5];
            for(int i=0;i<num;i++) {
                String[] app = (String[])list.get(i);
                apps[i] = app;
            }
           
            con.close();
            return apps;
        
        } catch(ClassNotFoundException ex) {
            ex.printStackTrace();
        } catch(SQLException ex) {
            ex.printStackTrace();
        }
        
        return new String[][] { };
    }    
    
    /**
     * Delete an appointment.
     * Authentication with username and password is required.
     * @return True if appointment was deleted successful otherwise FALSE.
     */
    @WebMethod
    public boolean deleteAppointment(@WebParam(name = "username") final String username, @WebParam(name = "password") final String password, @WebParam(name = "id") final String id) {
        
        Connection con;
        Statement  stat;
        try {
            
            Class.forName(dbDriver);
            con = DriverManager.getConnection(dbUrl,dbUser,dbPass);
            stat = con.createStatement();
            
            // Find USERID by username and password
            ResultSet result = stat.executeQuery("SELECT userid FROM users WHERE username='"+username+"' AND password='"+password+"';");
            if(result.next()) {
                String userID = result.getString("userid");
                System.out.println("USER FOUND:"+userID);
                stat = con.createStatement();
                int rows = stat.executeUpdate("DELETE FROM appointment WHERE appid='"+id+"' AND userid='"+userID+"';");
                con.commit();
                con.close();
                
                return rows>0;
            }
            
            // Username or Password not correct
            return false;
        } catch(Exception ex) {
            return false;
        }
    }

    /**
     * Simple Daytime Service.
     * Implemented for testing purpose.
     * It tells you the date and time on the server.
     */
    @WebMethod
    public String getTime() {
        return new Date().toString();
    }

    /**
     * Returns an Array of available Types.
     * A Type has a unique Type ID and a natural Name like "Meeting".
     * @return 2 Dimensional String Array
     */
    @WebMethod
    public String[][] getTypes() {
        ArrayList<String[]> list = new ArrayList<String[]>();
        
        try {
            
            Class.forName(dbDriver);
            Connection con = DriverManager.getConnection(dbUrl,dbUser,dbPass);
            Statement stat = con.createStatement();
            ResultSet result = stat.executeQuery("select TYPE,NAME from apptype");
            
            while(result.next()) {
                String[] type = new String[2];
                type[0] = result.getString("TYPE");
                type[1] = result.getString("NAME");
                
                list.add(type);
            }
            
            con.close();
            return list.toArray(new String[0][2]);
            
        } catch(Exception ex) {
            ex.printStackTrace();
        }
        
        return new String[][] {{}};
    }

    /**
     * Create a new Appointment Type.
     * An Appointment Type consists of a Name and a unique type ID.
     * The type ID is automatically generated, you dont have to worry about.
     * @return True if operation was successful. FALSE if the type already exists or in case of an exception.
     */
    @WebMethod
    public boolean createType(@WebParam(name = "name") final String name) {
        
        if(name!=null && !name.equals("")) {
            try {
                Class.forName(dbDriver);
                Connection con = DriverManager.getConnection(dbUrl,dbUser,dbPass);
                
                // Lookup if a Type with this name already exists
                Statement stat = con.createStatement();
                ResultSet result = stat.executeQuery("SELECT COUNT(name) AS ct FROM types WHERE name='"+name+"';");
                result.next();
                if(result.getInt("ct")==0) {
                    UUID guid = UUID.randomUUID();
                    String typeId = guid.toString().replaceAll("-","");
                    stat = con.createStatement();
                    stat.execute("INSERT INTO apptype(TYPE,NAME) values('"+typeId+"','"+name+"');");
                    con.commit();
                    con.close();
                    return true;
                }
                return false;
                        
            } catch(Exception ex) {
                ex.printStackTrace();
                return false;
            }
        } 
        return false;
    }

    /**
     * Delete an appointment type.
     * @params typeId The appointment type ID.
     * @see getTypes()
     */
    @WebMethod
    public boolean deleteType(@WebParam(name = "typeId") final String typeId) {
    
        Connection      con;
        Statement       stat;
        int             numRows;
        
        // TypeID is mandatory *what a surprise*
        if(typeId==null || typeId.equals(""))
            return false;
        
        try {
            Class.forName(dbDriver);
            con = DriverManager.getConnection(dbUrl,dbUser,dbPass);
            stat = con.createStatement();
            
            numRows = stat.executeUpdate("DELETE FROM apptype WHERE type='"+typeId+"';");
            return numRows>0;   // delete operation on one entry ?
            
        } catch(ClassNotFoundException ex) {
            ex.printStackTrace();
            return false;
        } catch(SQLException ex) {
            ex.printStackTrace();
            return false;
        }
   }



    
}
